@@ -6,7 +6,7 @@ Plot ancestry PCA analysis results
 
 import logging
 from collections import Counter
-from typing import List, Iterable
+from typing import List, Iterable, Optional
 
 import click
 import pandas as pd
@@ -74,7 +74,7 @@ def main(  # pylint: disable=too-many-arguments,too-many-locals,missing-function
     loadings_ht_path: str,
     provided_pop_ht_path: str,
     inferred_pop_ht_path: str,
-    out_path_pattern: str,
+    out_path_pattern: Optional[str],
     hail_billing: str,  # pylint: disable=unused-argument
 ):
     utils.init_hail(__file__)
@@ -95,7 +95,8 @@ def produce_plots(
     loadings_ht_path: str,
     provided_pop_ht_path: str,
     inferred_pop_ht_path: str,
-    out_path_pattern: str,
+    out_path_pattern: Optional[str] = None,
+    number_of_pcs: Optional[int] = None,
 ):
     """
     Generate plots in png and html formats, write for each PC (of n_pcs) and
@@ -135,17 +136,78 @@ def produce_plots(
     eigenvalues = pd.to_numeric(eigenvalues)
     variance = np.divide(eigenvalues[1:], float(eigenvalues.sum())) * 100
     variance = variance.round(2)
-    number_of_pcs = len(eigenvalues) - 1
+    number_of_pcs = number_of_pcs or len(eigenvalues) - 1
+
+    plots = []
 
     studies = scores.study.collect()
+    unique_studies = remove_duplicates(studies)
+    bg_studies = [s for s in unique_studies if s == 'gnomad']
+    fg_studies = [s for s in unique_studies if s != 'gnomad']
 
+    plots.extend(
+        _plot_study(
+            number_of_pcs,
+            variance,
+            scores,
+            sample_names,
+            unique_studies,
+            bg_studies,
+            fg_studies,
+            out_path_pattern=out_path_pattern,
+        )
+    )
+
+    plots.extend(
+        _plot_continental_pop(
+            number_of_pcs,
+            variance,
+            scores,
+            sample_names,
+            bg_studies,
+            fg_studies,
+            out_path_pattern=out_path_pattern,
+        )
+    )
+
+    plots.extend(
+        _plot_subcontinental_pop(
+            number_of_pcs,
+            variance,
+            scores,
+            sample_names,
+            bg_studies,
+            fg_studies,
+            out_path_pattern=out_path_pattern,
+        )
+    )
+
+    plots.extend(
+        _plot_loadings(
+            number_of_pcs, loadings_ht_path, out_path_pattern=out_path_pattern
+        )
+    )
+
+    return plots
+
+
+def _plot_study(
+    number_of_pcs,
+    variance,
+    scores,
+    studies,
+    sample_names,
+    bg_studies,
+    fg_studies,
+    out_path_pattern=None,
+):
     tooltips = [('labels', '@label'), ('samples', '@samples')]
-
     # plot by study
     cntr: Counter = Counter(studies)
     labels = [f'{x} ({cntr[x]})' for x in studies]
     unique_labels = list(Counter(labels).keys())
 
+    plots = []
     for i in range(number_of_pcs - 1):
         pc1 = i
         pc2 = i + 1
@@ -163,34 +225,51 @@ def produce_plots(
                 samples=sample_names,
             )
         )
-        plot.circle(
+        plot.scatter(
             'x',
             'y',
             alpha=0.5,
+            marker=factor_mark(
+                'study',
+                ['cross'] * len(bg_studies) + ['circle'] * len(fg_studies),
+                bg_studies + fg_studies,
+            ),
             source=source,
             size=4,
             color=factor_cmap('label', ['#1b9e77', '#d95f02'], unique_labels),
             legend_group='label',
         )
         plot.add_layout(plot.legend[0], 'left')
-        plot_filename = out_path_pattern.format(scope='study', pci=pc2, ext='png')
-        with hl.hadoop_open(plot_filename, 'wb') as f:
-            get_screenshot_as_png(plot).save(f, format='PNG')
-        html = file_html(plot, CDN, 'my plot')
-        plot_filename_html = out_path_pattern.format(scope='study', pci=pc2, ext='html')
-        with hl.hadoop_open(plot_filename_html, 'w') as f:
-            f.write(html)
+        plots.append(plot)
+        if out_path_pattern:
+            plot_filename = out_path_pattern.format(scope='study', pci=pc2, ext='png')
+            with hl.hadoop_open(plot_filename, 'wb') as f:
+                get_screenshot_as_png(plot).save(f, format='PNG')
+            html = file_html(plot, CDN, 'my plot')
+            plot_filename_html = out_path_pattern.format(
+                scope='study', pci=pc2, ext='html'
+            )
+            with hl.hadoop_open(plot_filename_html, 'w') as f:
+                f.write(html)
+    return plots
 
-    # plot by continental population
+
+def _plot_continental_pop(
+    number_of_pcs,
+    variance,
+    scores,
+    sample_names,
+    bg_studies,
+    fg_studies,
+    out_path_pattern=None,
+):
+    tooltips = [('labels', '@label'), ('samples', '@samples')]
     labels = scores.continental_pop.collect()
     cntr = Counter(labels)
     labels = [f'{x} ({cntr[x]})' for x in labels]
     unique_labels = list(Counter(labels).keys())
 
-    unique_studies = remove_duplicates(studies)
-    bg_studies = [s for s in unique_studies if s == 'gnomad']
-    fg_studies = [s for s in unique_studies if s != 'gnomad']
-
+    plots = []
     for i in range(number_of_pcs - 1):
         pc1 = i
         pc2 = i + 1
@@ -223,24 +302,38 @@ def produce_plots(
             legend_group='label',
         )
         plot.add_layout(plot.legend[0], 'left')
-        plot_filename = out_path_pattern.format(
-            scope='continental_pop', pci=pc2, ext='png'
-        )
-        with hl.hadoop_open(plot_filename, 'wb') as f:
-            get_screenshot_as_png(plot).save(f, format='PNG')
-        html = file_html(plot, CDN, 'my plot')
-        plot_filename_html = out_path_pattern.format(
-            scope='continental_pop', pci=pc2, ext='html'
-        )
-        with hl.hadoop_open(plot_filename_html, 'w') as f:
-            f.write(html)
+        plots.append(plot)
+        if out_path_pattern:
+            plot_filename = out_path_pattern.format(
+                scope='continental_pop', pci=pc2, ext='png'
+            )
+            with hl.hadoop_open(plot_filename, 'wb') as f:
+                get_screenshot_as_png(plot).save(f, format='PNG')
+            html = file_html(plot, CDN, 'my plot')
+            plot_filename_html = out_path_pattern.format(
+                scope='continental_pop', pci=pc2, ext='html'
+            )
+            with hl.hadoop_open(plot_filename_html, 'w') as f:
+                f.write(html)
+    return plots
 
-    # plot by subpopulation
+
+def _plot_subcontinental_pop(
+    number_of_pcs,
+    variance,
+    scores,
+    sample_names,
+    bg_studies,
+    fg_studies,
+    out_path_pattern=None,
+):
+    tooltips = [('labels', '@label'), ('samples', '@samples')]
     labels = scores.subpop.collect()
     cntr = Counter(labels)
     labels = [f'{x} ({cntr[x]})' for x in labels]
     unique_labels = list(Counter(labels).keys())
 
+    plots = []
     for i in range(number_of_pcs - 1):
         pc1 = i
         pc2 = i + 1
@@ -258,7 +351,7 @@ def produce_plots(
                 samples=sample_names,
             )
         )
-        plot.circle(
+        plot.scatter(
             'x',
             'y',
             alpha=0.6,
@@ -273,18 +366,23 @@ def produce_plots(
             legend_group='label',
         )
         plot.add_layout(plot.legend[0], 'left')
-        plot_filename = out_path_pattern.format(scope='subpop', pci=pc2, ext='png')
-        with hl.hadoop_open(plot_filename, 'wb') as f:
-            get_screenshot_as_png(plot).save(f, format='PNG')
-        html = file_html(plot, CDN, 'my plot')
-        plot_filename_html = out_path_pattern.format(
-            scope='subpop', pci=pc2, ext='html'
-        )
-        with hl.hadoop_open(plot_filename_html, 'w') as f:
-            f.write(html)
+        plots.append(plot)
+        if out_path_pattern:
+            plot_filename = out_path_pattern.format(scope='subpop', pci=pc2, ext='png')
+            with hl.hadoop_open(plot_filename, 'wb') as f:
+                get_screenshot_as_png(plot).save(f, format='PNG')
+            html = file_html(plot, CDN, 'my plot')
+            plot_filename_html = out_path_pattern.format(
+                scope='subpop', pci=pc2, ext='html'
+            )
+            with hl.hadoop_open(plot_filename_html, 'w') as f:
+                f.write(html)
+    return plots
 
-    # Plot loadings
+
+def _plot_loadings(number_of_pcs, loadings_ht_path, out_path_pattern=None):
     loadings_ht = hl.read_table(loadings_ht_path)
+    plots = []
     for i in range(number_of_pcs - 1):
         pc = i + 1
         plot = manhattan_loadings(
@@ -293,15 +391,18 @@ def produce_plots(
             title='Loadings of PC ' + str(pc),
             collect_all=True,
         )
-        plot_filename = out_path_pattern.format(scope='loadings', pci=pc, ext='png')
-        with hl.hadoop_open(plot_filename, 'wb') as f:
-            get_screenshot_as_png(plot).save(f, format='PNG')
-        html = file_html(plot, CDN, 'my plot')
-        plot_filename_html = out_path_pattern.format(
-            scope='loadings', pci=pc, ext='html'
-        )
-        with hl.hadoop_open(plot_filename_html, 'w') as f:
-            f.write(html)
+        plots.append(plot)
+        if out_path_pattern:
+            plot_filename = out_path_pattern.format(scope='loadings', pci=pc, ext='png')
+            with hl.hadoop_open(plot_filename, 'wb') as f:
+                get_screenshot_as_png(plot).save(f, format='PNG')
+            html = file_html(plot, CDN, 'my plot')
+            plot_filename_html = out_path_pattern.format(
+                scope='loadings', pci=pc, ext='html'
+            )
+            with hl.hadoop_open(plot_filename_html, 'w') as f:
+                f.write(html)
+    return plots
 
 
 def manhattan_loadings(
